# -*- coding: utf-8 -*-

import re

from .. import ValidationError

_regexes = {
    'word_notword_spaces': re.compile(r'[\w\W\s]*', re.IGNORECASE),
    'notword': re.compile(r'\W', re.IGNORECASE),
    'punctuation': re.compile(r'[^\s\w\-]+', re.IGNORECASE),
    'alphanumeric': re.compile(r'[a-z\s\-_0-9]*', re.IGNORECASE),
    'alpha': re.compile(r'[a-z\s\-_]*', re.IGNORECASE),
    'numeric': re.compile(r'\d*', re.IGNORECASE),
    'dash_under_space': re.compile(r'[\-_\s]+', re.IGNORECASE)
}


def _prehook(_strip=True, _value=None, *args, **kwargs):
    if _value is None:
        return u''

    if type(_value) not in (unicode, str):
        raise ValidationError('This _value must be a str or unicode type.')

    if _strip:
        _value = strip(_value=_value)

    return _value if type(_value) is unicode else unicode(_value)


def text(min_len=None, max_len=None, _value=None, *args, **kwargs):
    if min_len is not None and len(_value) <= min_len:
        raise ValidationError('Text length must be greater than %s' % min_len)

    if max_len is not None and len(_value) >= max_len:
        raise ValidationError('Text length must be less than %s' % max_len)

    return _value


def alphanumeric(_value=None, *args, **kwargs):
    if not _regexes['alphanumeric'].match(_value):
        raise ValidationError(
            'This _value must contain alphanumeric characters only.')

    return _value

alnum = alphanumeric


def alpha(_value=None, *args, **kwargs):
    if not _regexes['alpha'].match(_value):
        raise ValidationError(
            'This _value must contain alphabetical characters only.')

    return _value


def numeric_string(_value=None, *args, **kwargs):
    if not _regexes['numeric'].match(_value):
        raise ValidationError(
            'This _value must contain numeric characters only.')

    return _value


def nonblank(_value=None, *args, **kwargs):
    if len(_value) <= 0:
        raise ValidationError('This _value must not be empty.')

    return _value


def removespaces(_value=None, *args, **kwargs):
    return _value.replace(u' ', u'')


def strip(_value=None, *args, **kwargs):
    return _value.strip()


def lower(_value=None, *args, **kwargs):
    return _value.lower()


def upper(_value=None, *args, **kwargs):
    return _value.upper()


def regex(regex, case=False, _value=None, *args, **kwargs):
    if kwargs.get('case'):
        regex = re.compile(regex)
    else:
        regex = re.compile(regex, re.IGNORECASE)

    if not regex.match(_value):
        raise ValidationError('The _value must match the regex %s' % regex)

    return _value


def canonize(_value=None, *args, **kwargs):
    return _regexes['dash_under_space'].sub('_', _value.strip().lower())


def slugify(_value=None, *args, **kwargs):
    _value = _regexes['punctuation'].sub('', _value)
    _value = _regexes['notword'].sub('-', _value)
    return _value.lower()


def key_lookup(ref, nomatch='pass', _value=None, *args, **kwargs):
    try:
        return ref[_value]
    except KeyError:
        if nomatch == 'pass':
            return _value
        else:
            raise ValidationError(
                'Could not find a matching entry for key %s' % _value)



# -*- coding: utf-8 -*-

from nose.tools.trivial import assert_equals, assert_true, assert_false, assert_is, assert_is_instance

from datetime import date, time, datetime

from valhalla import Schema

test_data = {
	'valid_date': '12/06/1989',
	'valid_europe_date': '06/12/1898',
	'valid_europe_expanded': '06 Dec 1989',
	'invalid_date': '09/31/2013', # does not have 31 days
	'garbage_date': '19922-12 01',
	'valid_time': '12:06:24',
	'valid_24_time': '13:01:01',
	'invalid_12_time': '22:00:00',
	'invalid_24_time': '26:00:00',
	'garbage_time': '46:06- 24',
	'valid_datetime': '12/06/1989 12:06:24',
	'invalid_datetime': '09/31/2013 12:06:24',
	'my_birthday': '12/06/1989',
	'my_favorite_time': '04:20:00',
	'far_far_away': '2050-12-06',
	'long_time_ago': '1800-01-01'
}

def test_date():
	s = Schema()
	s.valid_date.date()
	s.valid_europe_date.date(format='europe')
	s.valid_europe_expanded.date(format='europe_expanded')
	s.invalid_date.date()
	s.garbage_date.date()

	s.validate(test_data)
	assert_false(s.valid)

	assert_true(s.valid_date.valid)
	assert_true(s.valid_europe_date.valid)
	assert_true(s.valid_europe_expanded.valid)
	assert_false(s.invalid_date.valid)
	assert_false(s.garbage_date.valid)

def test_time():
	s = Schema()
	s.valid_time.time()
	s.valid_24_time.time(format='military')
	s.invalid_12_time.time()
	s.invalid_24_time.time(format='military')
	s.garbage_time.time()

	s.validate(test_data)
	assert_false(s.valid)

	assert_true(s.valid_time.valid)
	assert_true(s.valid_24_time.valid)
	assert_false(s.invalid_12_time.valid)
	assert_false(s.invalid_24_time.valid)
	assert_false(s.garbage_time.valid)

def test_datetime():
	s = Schema()
	s.valid_datetime.datetime()
	s.invalid_datetime.datetime()

	s.validate(test_data)
	assert_false(s.valid)

	assert_true(s.valid_datetime.valid)
	assert_false(s.invalid_datetime.valid)

def test_time_before():
	s = Schema()
	s.my_birthday.date().time_before(date.today())
	s.far_far_away.date(format='american_scientific').time_before(date.today())
	s.long_time_ago.date(format='american_scientific').time_before(date.today())

	s.validate(test_data)
	assert_false(s.valid)

	assert_true(s.my_birthday.valid)
	assert_false(s.far_far_away.valid) # one day this assertion will be True :)
	assert_true(s.long_time_ago.valid)

def test_time_after():
	s = Schema()
	s.my_birthday.date().time_after(date.today())
	s.far_far_away.date(format='american_scientific').time_after(date.today())
	s.long_time_ago.date(format='american_scientific').time_after(date.today())

	s.validate(test_data)
	assert_false(s.valid)

	assert_false(s.my_birthday.valid)
	assert_true(s.far_far_away.valid)
	assert_false(s.long_time_ago.valid)

def test_time_between():
	s = Schema()
	s.my_birthday.date().time_between(date(1988, 1, 1), date.today())
	s.long_time_ago.date(format='american_scientific').time_between(date.today(), date(2055, 1, 1))

	s.validate(test_data)
	assert_false(s.valid)

	assert_true(s.my_birthday.valid)
	assert_false(s.long_time_ago.valid)


from nose.tools.trivial import (assert_equals, assert_true)

from valhalla import Schema

test_data = {
    'some_dict': {'one': 'fish', 'two': 'fish', 'red': 'fish', 'blue': 'fish'},
    'some_list': ['alpha', 'bravo', 'charlie', 'delta'],
    'some_other_list': ['alpha', 'bravo', 'charlie']
}


def test_drop_keys():
    s = Schema()
    s.some_dict.drop_keys('one', 'two')

    s.validate(test_data)
    assert_true(s.valid)

    assert_equals(s.some_dict.result, {'red': 'fish', 'blue': 'fish'})


def test_contains():
    s = Schema()
    s.some_list.contains('bravo')
    s.some_other_list.contains('alpha', 'bravo')

    s.validate(test_data)
    assert_true(s.valid)



# -*- coding: utf-8 -*-

from nose.tools.trivial import assert_true, assert_false

from valhalla import Schema


def test_range():
    s = Schema()
    s.some_number.range()
    s.validate({'some_number': 4})
    assert_true(s.valid)

    s = Schema()
    s.some_number.range(low=1)
    s.validate({'some_number': 0})
    assert_false(s.valid)
    s.reset()
    s.validate({'some_number': 1})
    assert_true(s.valid)
    s.reset()
    s.validate({'some_number': 2})
    assert_true(s.valid)

    s = Schema()
    s.some_number.range(high=5)
    s.validate({'some_number': 6})
    assert_false(s.valid)
    s.reset()
    s.validate({'some_number': 4})
    assert_true(s.valid)

    s = Schema()
    s.some_number.range(low=2, high=3)
    s.validate({'some_number': 2})
    assert_true(s.valid)


def test_minimum():
    s = Schema()
    s.some_number.minimum(1)
    s.validate({'some_number': 0})
    assert_false(s.valid)
    s.reset()
    s.validate({'some_number': 1})
    assert_true(s.valid)


def test_maximum():
    s = Schema()
    s.some_number.maximum(5)
    s.validate({'some_number': 6})
    assert_false(s.valid)
    s.reset()
    s.validate({'some_number': 5})
    assert_true(s.valid)


def test_between():
    s = Schema()
    s.some_number.between(5, 10)
    s.validate({'some_number': 5})
    assert_false(s.valid)
    s.reset()
    s.validate({'some_number': 6})
    assert_true(s.valid)
    s.reset()
    s.validate({'some_number': 10})
    assert_false(s.valid)


def test_equal():
    s = Schema()
    s.some_number.equal(10)
    s.validate({'some_number': 11})
    assert_false(s.valid)
    s.reset()
    s.validate({'some_number': 10})
    assert_true(s.valid)


def test_zero():
    s = Schema()
    s.some_number.zero()
    s.validate({'some_number': 1})
    assert_false(s.valid)
    s.reset()
    s.validate({'some_number': 0})
    assert_true(s.valid)



# -*- coding: utf-8 -*-

import re

from . import strings
from .. import ValidationError

_regexes = {
	'email_address': re.compile(r'^[a-z0-9\._\+%-]+@[a-z0-9\.-]+(\.[A-Z]{2,4})+$', re.IGNORECASE),
	'ipv4': re.compile(r'^([0-9]{1,3}\.){3}[0-9]{1,3}$'),
	'url': re.compile(r'')
}

# debating breaking ALL inter-module dependencies, 
# including this seemingly innocuous reference.
def _prehook(*args, **kwargs):
	return strings._prehook(_value=kwargs.get('_value'))

def email(_value=None, *args, **kwargs):
	if not _regexes['email_address'].match(_value):
		raise ValidationError('This value must be a valid email address.')
	return _value

def ipv4(_value=None, *args, **kwargs):
	if not _regexes['ipv4'].match(_value):
		raise ValidationError('This value must be a valid IPV4 address.')

	if filter(lambda x: int(x) > 255, _value.split('.')):
		raise ValidationError('Octets may not exceed 255.')

	return _value

def cidr4(_value=None, *args, **kwargs): pass
def cidr6(_value=None, *args, **kwargs): pass
def macaddress(_value=None, *args, **kwargs): pass

# this is really just a shortcut for uri(schemes=['http', 'https'])
def url(scheme=False, tld=True, qs=True, frag=True, _value=None, *args, **kwargs): pass

# will implement this when I feel like pasting in a giant list of registered schemes...
def uri(schemes=[], _value=None, *args, **kwargs): pass



# -*- coding: utf-8 -*-

from nose.tools.trivial import (
    assert_equals, assert_true, assert_false,
    assert_is_instance
)

from valhalla import Schema

test_data = {
    'true_value': 'i am truth',
    'false_value': '',
    'str_false_1': 'false',
    'str_false_2': 'undefined',
    'str_false_3': '0',
    'str_false_4': '[]',
    'str_false_5': '{}',
    'str_false_6': 'None',
    'integer_value': '54',
    'float_value': '4.23',
    'not_integer': 'monkey time',
    'sizable_number': '12390858430439843049084325754382940234',
    'decimal_value': '4.5',
    'not_decimal_value': '4',
    'string_value': 'i am a string'
}


def test_boolean():
    s = Schema(blank='all')
    s.true_value.boolean()
    s.false_value.boolean()
    s.validate(test_data)
    assert_true(s.valid)
    assert_true(s.true_value.result)
    assert_equals(s.false_value.result, False)


def test_strbool():
    s = Schema(blank='all')
    s.true_value.strbool()
    s.str_false_1.strbool()
    s.str_false_2.strbool()
    s.str_false_3.strbool()
    s.str_false_4.strbool()
    s.str_false_5.strbool()
    s.str_false_6.strbool()

    s.validate(test_data)
    assert_true(s.valid)

    assert_true(s.true_value.result)
    assert_equals(s.str_false_1.result, False)
    assert_equals(s.str_false_2.result, False)
    assert_equals(s.str_false_3.result, False)
    assert_equals(s.str_false_4.result, False)
    assert_equals(s.str_false_5.result, False)
    assert_equals(s.str_false_6.result, False)


def test_integer():
    s = Schema()
    s.integer_value.integer()
    s.float_value.integer()
    s.not_integer.integer()

    s.validate(test_data)
    assert_false(s.valid)

    assert_true(s.integer_value.valid)
    assert_false(s.float_value.valid)
    assert_false(s.not_integer.valid)

    assert_is_instance(s.integer_value.result, int)


def test_longint():
    s = Schema()
    s.integer_value.longint()
    s.float_value.longint()
    s.sizable_number.longint()

    s.validate(test_data)
    assert_false(s.valid)

    assert_is_instance(s.integer_value.result, long)
    assert_is_instance(s.sizable_number.result, long)


def test_numeric():
    s = Schema()
    s.integer_value.numeric()
    s.float_value.numeric()

    s.validate(test_data)
    assert_true(s.valid)

    assert_is_instance(s.integer_value.result, int)
    assert_is_instance(s.float_value.result, float)


def test_decimal():
    s = Schema()
    s.decimal_value.decimal()
    s.not_decimal_value.decimal()
    s.string_value.decimal()

    s.validate(test_data)
    assert_false(s.valid)

    assert_true(s.decimal_value.valid)
    assert_true(s.not_decimal_value.valid)
    assert_false(s.string_value.valid)


def test_string():
    pass


def test_none():
    pass



# -*- coding: utf-8 -*-

from decimal import Decimal, InvalidOperation
from valhalla import ValidationError


def boolean(_value=None, *args, **kwargs):
    return False if not _value else True


def strbool(_value=None, *args, **kwargs):
    false_list = [
        'false',
        '0'
    ]

    return False if not _value or _value in false_list else True


def integer(_value=None, *args, **kwargs):
    try:
        return int(_value)
    except ValueError:
        raise ValidationError(
            'The value %s cannot be converted to an integer.' % _value)


def longint(_value=None, *args, **kwargs):
    try:
        return long(_value)
    except ValueError:
        raise ValidationError(
            'The value %s cannot be converted to a long integer.' % _value)


def numeric(_value=None, *args, **kwargs):
    if float(_value).is_integer():
        return int(_value)

    return float(_value)


def decimal(_value=None, *args, **kwargs):
    try:
        return Decimal(_value)
    except (ValueError, InvalidOperation):
        raise ValidationError(
            'The specified value %s could not be casted to a Decimal' % _value)


def string(_value=None, *args, **kwargs):
    try:
        return unicode(_value)
    except ValueError:
        raise ValidationError(
            'The specified value %s could not be casted to a unicode string' % _value)


def none(_value=None, *args, **kwargs):
    none_list = [
        '',
        'none',
        'undefined',
        'null',
        'n/a',
        'na',
        '[]',
        '{}',
    ]

    value = str(_value).lower().strip()

    return None if value in none_list else _value



# -*- coding: utf-8 -*-

from functools import partial, wraps


class ValidationError(Exception):
    pass


def _field_option(option_name):
    def _decorator(f):
        @wraps(f)
        def _wrapper(*args, **kwargs):
            self, field_names = args[0], args[1]

            if self._field_options[option_name] is not None:
                self._field_options[option_name] = None

                fields = []
                field_groups = []

                args = list(args)
                if option_name == 'match':
                    for g in field_names:
                        field_groups.append(self._get_fields_by_name(g))
                    args.append(field_groups)
                else:
                    if type(field_names) is list:
                        fields = self._get_fields_by_name(field_names)
                    elif field_names == 'all':
                        fields = self._fields.values()
                    args.append(fields)
                args = tuple(args)
                return f(*args, **kwargs)
            return None
        return _wrapper
    return _decorator


class Schema(object):

    @classmethod
    def from_dict(cls, dict_scheme, **kwargs):
        if type(dict_scheme) is not dict:
            raise TypeError(
                'Dict scheme must be dict and follow valid format.')

        s = Schema(**kwargs)
        for f_name, modifiers in dict_scheme.iteritems():
            field = getattr(s, f_name)
            for m in modifiers:
                if type(m) is str:
                    getattr(field, m)()
                elif type(m) is tuple:
                    getattr(field, m[0])(*m[1:])
        return s

    def __init__(self, name=None, match=[], require=[], blank=[],
                 extra='ignore', force_unicode=True):
        self._name = unicode(name)
        self._fields = {}
        self._extra = extra
        self._missing = []
        self._force_unicode = force_unicode

        self._field_options = {}
        self._field_options['match'] = match
        self._field_options['require'] = require
        self._field_options['blank'] = blank

    def __getattr__(self, attr):
        return self.add_field(attr, Field(self, attr))

    def __repr__(self):
        return '<%r: [%r] >' % (self.name, ', '.join(
            [f for f in self._fields.values()]))

    def _get_fields_by_name(self, field_names):
        fields = []
        for f_name in field_names:
            try:
                fields.append(self._fields[unicode(f_name)])
            except KeyError:
                pass
        return set(fields)

    @property
    def schema_name(self):
        return self._name

    @property
    def errors(self):
        return {f.name: f.errors for f_name, f in self._fields.iteritems()}

    @property
    def valid(self):
        return all([f.valid for name, f in self._fields.iteritems()])

    @property
    def missing(self):
        return list(self._missing)

    @property
    def results(self):
        return {f.name: f.result for n, f in self._fields.iteritems()
                if f.valid}

    def add_field(self, name, field):
        name = unicode(name)
        self._fields[name] = field
        setattr(self, name, field)
        return field

    @_field_option('require')
    def required_fields(self, field_names, fields):
        [f.require(True) for f in fields]

    def optional_fields(self, field_names):
        fields = self._get_fields_by_name(field_names)
        [f.require(False) for f in fields]

    @_field_option('blank')
    def blank_fields(self, field_names, fields):
        [f.blank(True) for f in fields]

    def not_blank_fields(self, field_names):
        fields = self._get_fields_by_name(field_names)
        [f.blank(False) for f in fields]

    @_field_option('match')
    def match_fields(self, field_names, field_groups):
        for group in field_groups:
            for f in group:
                others = group.copy()
                others.discard(f)
                f.match(*others)

    def validate(self, data_dict, **kwargs):
        supplied_fields = set(data_dict.keys())
        actual_fields = set(self._fields.keys())
        known_fields = supplied_fields.intersection(actual_fields)
        #unknown_fields = supplied_fields.difference(actual_fields)

        if self._extra == 'ignore':
            data_dict = {k: data_dict[k] for k in known_fields}

        if self._force_unicode:
            data_dict = self._cast_to_unicode(data_dict)

        self.required_fields(self._field_options['require'])
        self.blank_fields(self._field_options['blank'])
        self.match_fields(self._field_options['match'])

        self._validate_required(data_dict)
        self._validate_blank(data_dict)
        self._validate_matching(data_dict)

        [field.validate(data_dict.get(f_name))
         for f_name, field in self._fields.iteritems()]

    def _cast_to_unicode(self, data_dict):

        def _string_cast(value):
            if type(value) is str:
                return unicode(value)
            return value

        new_dict = {}

        for k, v in data_dict.iteritems():
            new_k = _string_cast(k)
            if type(v) is (set):
                v = dict(v)
            if type(v) is (dict):
                new_dict[new_k] = self._cast_to_unicode(v)
                continue
            elif type(v) is list:
                new_dict[new_k] = [_string_cast(i) for i in v]
            elif type(v) is tuple:
                new_dict[new_k] = tuple([_string_cast(i) for i in v])
            else:
                new_dict[new_k] = _string_cast(v)
        return new_dict

    def _validate_required(self, data_dict):
        required_fields = set(
            [f.name for f_name, f in self._fields.iteritems() if f.required])
        supplied_fields = set(data_dict.keys())

        missing_fields = required_fields - supplied_fields

        s = set(self._get_fields_by_name(supplied_fields))
        m = set(self._get_fields_by_name(missing_fields))
        actually_missing = m - s

        for f in actually_missing:
            f._ran = True
            f._errors.append('This field cannot be missing.')

        return not actually_missing

    def _validate_blank(self, data_dict):
        for f_name, value in data_dict.iteritems():
            field = self._fields[f_name]

            if none(value) is None:
                if not field.blank_allowed:
                    field._errors.append('This field cannot be blank.')
                    field._ran = True

    def _validate_matching(self, data_dict):
        for f_name, value in data_dict.iteritems():
            field = self._fields[f_name]
            must_match = field.must_match

            if must_match and type(must_match[0]) is not Field:
                must_match = self._get_fields_by_name(must_match)

            if any([value != data_dict[m.name] for m in must_match]):
                for m in must_match:
                    m._ran = True
                field._ran = True
                field._errors.append(
                    'This field must match: [%s]' % ', '.join(
                        [m.name for m in must_match]))

    def reset(self):
        [f.reset() for n, f in self._fields.iteritems()]


class Field(object):

    def __init__(self, schema, name):
        self._schema = schema
        self._name = unicode(name)
        self._filters = []
        self._required = None
        self._blank = None
        self._alternate_fields = []
        self._match = []

        self.reset()

    def __repr__(self):
        return '<%r: (%r, %r)>' % (self.name, self.original, self.result)

    @property
    def name(self):
        return self._name

    @property
    def valid(self):
        return self._valid

    @property
    def errors(self):
        return self._errors

    @property
    def original(self):
        return self._original_value

    @property
    def result(self):
        return self._value

    @property
    def required(self):
        if self._required is None:
            return False
        return self._required

    @property
    def blank_allowed(self):
        if self._blank is None:
            return False
        return self._blank

    @property
    def must_match(self):
        return self._match

    def reset(self):
        self._ran = False
        self._valid = False
        self._errors = []
        self._original_value = self._value = None

    def validate(self, value):
        if self._ran:
            return self._valid

        self._original_value = self._value = none(_value=value)

        try:
            for f in self._filters:
                self._value = f.run(self._value)
        except ValidationError as e:
            self._valid = False
            self._errors.append(e.message)
        else:
            self._valid = True

        return self._valid

    def require(self, value=True):
        if self._required is None:
            self._required = value
        return self

    def blank(self, value=True):
        if self._blank is None:
            self._blank = value
        return self

    def match(self, *fields):
        if not self._match:
            self._match = fields
        return self

    def alt(self, alt_name):
        self._schema.add_field(alt_name, self)
        return self

    def __call__(self, *args, **kwargs):
        return self

    def __getattr__(self, name):
        fxn, pre, post = lookup(name)
        if not fxn:
            raise RuntimeError('Filter %s is undefined' % name)

        if pre:
            pref = Filter(self, pre)
            self._filters.append(pref)

        f = Filter(self, fxn)
        self._filters.append(f)

        if post:
            postf = Filter(self, post)
            self._filters.append(postf)

        return f


class Filter(object):

    def __init__(self, field, fxn):
        self._field = field
        self._validation_fxn = fxn

    def __call__(self, *args, **kwargs):
        partial_fxn = partial(self._validation_fxn, *args, **kwargs)
        partial_fxn.__name__ = self._validation_fxn.__name__
        self._validation_fxn = partial_fxn
        return self._field

    def __repr__(self):
        return '[Filter - %r]' % self._validation_fxn.__name__

    def run(self, value):
        processed = self._validation_fxn(_value=value)
        return processed


from filters import lookup
from filters.casting import none



# -*- coding: utf-8 -*-

from nose.tools.trivial import (assert_equals, assert_true, assert_false)

from valhalla import Schema

test_data = {
    'valid_visa': ('4012888888881881', '12.2020', '124'),
    'valid_amex': ('378282246310005', '09-2020', '9090'),
    'valid_mc': ('5105105105105100', '120', '123'),
    'valid_discover': ('6011000990139424', '120', '123'),
    'valid_other': ('30569309025904', '920', '123'),
    'invalid_number': ('6011000990139429', '920', '123'),
    'invalid_date': ('30569309025904', '12-06-1989', '124'),
    'invalid_csc': ('378282246310005', '120', '123'),
    'forbidden_type': ('378282246310005', '09-2020', '9090'),
    'expired_date': ('378282246310005', '0689', '9090')
}


def test_credit_card():
    s = Schema()

    s.valid_visa.credit_card()
    s.valid_amex.credit_card()
    s.valid_mc.credit_card()
    s.valid_discover.credit_card()
    s.valid_other.credit_card()
    s.invalid_number.credit_card()
    s.invalid_date.credit_card()
    s.invalid_csc.credit_card()
    s.forbidden_type.credit_card(brands=['visa'])
    s.expired_date.credit_card()

    s.validate(test_data)
    assert_false(s.valid)

    assert_true(s.valid_visa.valid)
    assert_true(s.valid_amex.valid)
    assert_true(s.valid_mc.valid)
    assert_true(s.valid_discover.valid)
    assert_true(s.valid_other.valid)
    assert_false(s.invalid_number.valid)
    assert_false(s.invalid_date.valid)
    assert_false(s.invalid_csc.valid)
    assert_false(s.forbidden_type.valid)
    assert_false(s.expired_date.valid)

    assert_equals(s.valid_visa.result[3], u'visa')
    assert_equals(s.valid_amex.result[3], u'amex')
    assert_equals(s.valid_mc.result[3], u'mastercard')
    assert_equals(s.valid_discover.result[3], u'discover')
    assert_equals(s.valid_other.result[3], u'other')

    assert_equals(s.invalid_number.errors[
                  0], 'The specified credit card number is not valid.')
    assert_equals(s.invalid_date.errors[
                  0], 'Invalid expiration date format. Must be one of : [MMYY, MYY, MYYYY, MMYYYY]')
    assert_equals(s.invalid_csc.errors[0], 'The CSC code is not valid.')
    assert_equals(s.forbidden_type.errors[
                  0], 'The type amex is not an acceptable brand.')
    assert_equals(s.expired_date.errors[0], 'The credit card is expired.')



# -*- coding: utf-8 -*-

from nose.tools.trivial import (assert_equals, assert_true, assert_false,
                                assert_in)


from valhalla import Schema

test_schema_data = {
    'blank_field': '',
    'other_blank_field': '',
    'required_field': 'I am required',
    'match_me': 'match.com',
    'match_to_me': 'match.com',
    'not_a_good_match': 'notmatch.com'
}

test_data_two = {
    'password': '1234',
    'password_confirm': '12345',
    'field_one': 'i am required',
    'can_be_blank': ''
}

test_required_data = {
    'required_field': 'Here I am',
    'required_field_alt': 'So am I'
}

test_dict_schema = {
    'email_address': ['email', ('alt', 'email'), 'require'],
    'some_number': ['numeric', ('range', 10, 15)],
    'some_collection': [('contains', 'apple')]
}

dict_schema_data = {
    'email': 'petermelias@gmail.com',
    'some_number': '13',
    'some_collection': ['orange', 'apple', 'monkey']
}


def test_schema():
    s = Schema()
    s.blank_field
    s.other_blank_field.blank(True)
    s.required_field.require()
    s.missing_field.require()
    s.match_me.match('match_to_me', 'not_a_good_match')
    s.match_to_me
    s.not_a_good_match

    s.validate(test_schema_data)
    assert_false(s.valid)

    assert_false(s.blank_field.valid)
    assert_true(s.other_blank_field.valid)
    assert_true(s.required_field.valid)
    assert_false(s.missing_field.valid)
    assert_false(s.match_me.valid)
    assert_false(s.match_to_me.valid)
    assert_false(s.not_a_good_match.valid)

    s = Schema(match=[('password', 'password_confirm')],
               require=['field_one', 'field_two'], blank=['can_be_blank'])
    s.password
    s.password_confirm
    s.field_one
    s.field_two
    s.can_be_blank

    s.validate(test_data_two)
    assert_false(s.valid)

    assert_false(s.password.valid)
    assert_false(s.password_confirm.valid)
    assert_true(s.field_one.valid)
    assert_false(s.field_two.valid)
    assert_true(s.can_be_blank.valid)

    assert_equals(
        s.results, {'field_one': 'i am required', 'can_be_blank': None})

    s = Schema()
    s.some_field.alt('required_field').require()
    s.validate(test_required_data)

    assert_true(s.valid)

    s = Schema.from_dict(test_dict_schema)
    s.validate(dict_schema_data)
    assert_true(s.valid)


def test_field():
    s = Schema('Test Schema')
    s.second_field().alt('other_name').require()
    d = {
        'other_name': 'Test Value A',
        'second_field': 'Test Value B'
    }
    s.validate(d)
    assert_true(s.valid)
    assert_equals(s.other_name.result, 'Test Value A')
    # because this should not have been run twice
    assert_equals(s.second_field.result, 'Test Value A')
    s.reset()
    assert_false(s.valid)

    d = {'total_fail': 'will not work'}
    s.validate(d)
    assert_false(s.valid)

    assert_false(s.other_name.valid)
    assert_false(s.second_field.valid)
    assert_in('This field cannot be missing.', s.other_name.errors)


def test_filter():
    s = Schema('Test Schema')
    s.first_name.text()
    s.last_name.text(min_len=1, max_len=10)
    d = {
        'first_name': 'Jack',
        'last_name': 'Bauer'
    }
    s.validate(d)

    assert_equals(s.first_name.original, 'Jack')
    assert_equals(s.first_name.result, u'Jack')

    assert_true(s.last_name.valid)

    d = {
        'last_name': ''
    }
    s.reset()
    s.validate(d)
    assert_false(s.valid),



# -*- coding: utf-8 -*-

from nose.tools.trivial import assert_equals, assert_true, assert_false

from valhalla import Schema

def _sample_data():
	return {
		'valid_email': 'petermelias@gmail.com',
		'invalid_email': 'fdsfdsf3343232@dsfdsfds',
		'invalid_email2': 123445455444,
		'invalid_email3': '1111111111111111111',
		'valid_ipv4': '127.0.0.1',
		'invalid_ipv4': '12333330',
		'invalid_ipv42': '1233.122.22.22'
	}

def test_email():
	s = Schema()
	s.valid_email.email()
	s.invalid_email.email()
	s.invalid_email2.email()
	s.invalid_email3.email()
	s.validate(_sample_data())

	assert_true(s.valid_email.valid)
	assert_false(s.invalid_email.valid)
	assert_false(s.invalid_email2.valid)
	assert_false(s.invalid_email3.valid)

def test_ipv4():
	s = Schema()
	s.valid_ipv4.ipv4()
	s.invalid_ipv4.ipv4()
	s.invalid_ipv42.ipv4()

	s.validate(_sample_data())

	assert_true(s.valid_ipv4.valid)
	assert_false(s.invalid_ipv4.valid)
	assert_false(s.invalid_ipv42.valid)


# -*- coding: utf-8 -*-

from nose.tools.trivial import assert_true, assert_equals

from valhalla import Schema


def test_constant():
	s = Schema()
	s.some_cst_value.constant('blue apple')

	s.validate({'some_cst_value': 'Roger that'})
	assert_true(s.valid)

	assert_equals(s.some_cst_value.result, 'blue apple')


# -*- coding: utf-8 -*-

from numbers import Number

from .. import ValidationError


def _prehook(_value=None, *args, **kwargs):
    if not isinstance(_value, Number):
        raise ValidationError('This _value must be a number.')

    return _value


def range(low=None, high=None, _value=None, *args, **kwargs):
    if low is not None and _value < low:
        raise ValidationError(
            'This number must be greater than or equal to %s' % low)

    if high is not None and _value > high:
        raise ValidationError(
            'This number must be less than or equal to %s' % high)

    return _value


def minimum(number, _value=None, *args, **kwargs):
    if _value < number:
        raise ValidationError(
            'This number must be greater than or equal to %s' % number)

    return _value


def maximum(number, _value=None, *args, **kwargs):
    if _value > number:
        raise ValidationError(
            'This number must be less than or equal to %s' % number)

    return _value


def between(low=None, high=None, _value=None, *args, **kwargs):
    if low is not None and _value <= low:
        raise ValidationError('This number must be greater than %s' % low)

    if high is not None and _value >= high:
        raise ValidationError('This number must be less than %s' % high)

    return _value


def equal(number, _value=None, *args, **kwargs):
    if _value != number:
        raise ValidationError('This number must be equal to %s' % number)

    return _value


def zero(_value=None, *args, **kwargs):
    if _value is not 0:
        raise ValidationError('This number must be zero.')
    return _value



# -*- coding: utf-8 -*-

from datetime import datetime as m_datetime, date as m_date, time as m_time, timedelta as m_timedelta

from .. import ValidationError

_common_date_formats = {
	'american': '%m/%d/%Y',
	'american_expanded': '%d %b %Y',
	'american_scientific': '%Y-%m-%d',
	'europe': '%d/%m/%Y',
	'europe_expanded': '%d %b %Y'
}

_common_time_formats = {
	'civilian': '%I:%M:%S',
	'military': '%H:%M:%S'
}

_common_datetime_formats = {
	'unix_timestamp': '%Y-%m-%d %H:%M:%S'
}

def _ensure_datetime(chrono_obj):
	if isinstance(chrono_obj, m_date):
		chrono_obj = m_datetime.combine(chrono_obj, m_time.max)
	elif isinstance(chrono_obj, m_time):
		chrono_obj = m_datetime.combine(m_date.today(), chrono_obj)
	elif not isinstance(chrono_obj, m_datetime):
		raise ValidationError('This method requires a chronological object of some kind: [date, time, datetime]')

	return chrono_obj

# attempts to return a valid Date object
def date(format='%m/%d/%Y', _value=None, *args, **kwargs):
	try:
		format = _common_date_formats[format]
	except KeyError: pass

	try:
		dt = m_datetime.strptime(_value, format)
		return dt.date()
	except ValueError:
		raise ValidationError('There was a problem converting the value %s to a valid date.' % _value)


# attempts to return a valid Time object
def time(format='%I:%M:%S', _value=None, *args, **kwargs):
	try:
		format = _common_time_formats[format]
	except KeyError: pass

	try:
		dt = m_datetime.strptime(_value, format)
		return dt.time()
	except ValueError:
		raise ValidationError('There was a problem converting the value %s to a valid time.' % _value)

def datetime(format='%m/%d/%Y %I:%M:%S', _value=None, *args, **kwargs):
	try:
		format = _common_datetime_formats[format]
	except KeyError: pass

	try:
		dt = m_datetime.strptime(_value, format)
		return dt
	except ValueError:
		raise ValidationError('There was a problem converting the value %s to a valid date time.' % _value)

# validates that datetime is before specified datetime
def time_before(deadline, _value=None, *args, **kwargs):
	_value = _ensure_datetime(_value)
	deadline = _ensure_datetime(deadline)
	if _value > deadline:
		raise ValidationError('The date %s has exceeded the deadline of %s' % (_value, deadline))
	return _value

# validates that datetime is after specified datetime
def time_after(milestone, _value=None, *args, **kwargs):
	_value = _ensure_datetime(_value)
	milestone = _ensure_datetime(milestone)
	if _value < milestone:
		raise ValidationError('The date %s occurs before the specified milestone of %s' % (_value, milestone))
	return _value

# validates that datetime is between specified times
def time_between(milestone, deadline, _value=None, *args, **kwargs):
	_value = _ensure_datetime(_value)
	milestone = _ensure_datetime(milestone)
	deadline = _ensure_datetime(deadline)
	if _value < milestone or _value > deadline:
		raise ValidationError('The specified date/time does not fall within the required date range.')
	return _value


# -*- coding: utf-8 -*-

from valhalla import ValidationError


def _prehook(_value=None, *args, **kwargs):
    if type(_value) not in (dict, list, set, tuple):
        raise ValidationError(
            'This validator requires a collection type [dict, list, set, tuple].')
    return _value


def drop_keys(*args, **kwargs):
    _value = kwargs.pop('_value')

    if type(_value) is not dict:
        raise ValidationError('This validator requires a dictionary.')
    [_value.pop(unicode(k), None) for k in args]
    return _value


def contains(*args, **kwargs):
    _value = kwargs.pop('_value')
    for k in args:
        if k not in _value:
            raise ValidationError(
                'The key %s was not found in the target collection.' % k)
    return _value



# -*- coding: utf-8 -*-

from nose.tools.trivial import assert_equals, assert_true, assert_false

from valhalla import Schema


def _sample_data():
    return {
        'blank_string': '',
        'none_type': None,
        'alpha_string': 'abcdefgh',
        'alnum_string': 'abc123',
        'numeric_string': '123456',
        'random_string': '1232 fsdfdsfs 993132.....fdsfd#@#@KLJFD(((#@)_',
        'actual_numbers': 1234566779999,
        'awkward_spaces': 'open   the    door get on the    floo   rrr',
        'flanking_spaces': '        everybody walk the dinosaur      ',
        'upper_case': 'I AM THE RAWRMACHINE',
        'lower_case': 'i am the quiet mouse',
        'noncanonical_string': 'THERE IS NOTHING-CANONICAL__ABOUTMEEEEEE333            see?',
        'notslugified': 'I AM NOT SLUGIFIED #@(#*@(#@ OMG PUNCTU3232...ATION    ---- who submits data like this anyway?',
        'some_key': 'alpha',
        'some_nonexistent_key': 'charlie',
        'some_nonexistent_key_2': 'delta'
    }


def test_text():
    s = Schema()
    s.random_string.text()
    s.validate(_sample_data())

    assert_true(s.valid)

    s = Schema()
    s.actual_numbers.text()

    s.validate(_sample_data())
    assert_false(s.valid)


def test_alnum():
    s = Schema()
    s.alnum_string.alnum()
    s.validate(_sample_data())

    assert_true(s.valid)

    s = Schema()
    s.alpha_string.alnum()
    s.actual_numbers.alnum()

    s.validate(_sample_data())

    assert_false(s.valid)
    assert_false(s.actual_numbers.valid)
    assert_true(s.alpha_string.valid)


def test_alpha():
    s = Schema()
    s.alpha_string.alpha()
    s.validate(_sample_data())

    assert_true(s.valid)

    s = Schema()
    s.actual_numbers.alpha()
    s.alnum_string.alpha()

    assert_false(s.valid)
    assert_false(s.actual_numbers.valid)
    assert_false(s.alnum_string.valid)


def test_numeric_string():
    s = Schema()
    s.numeric_string.numeric_string()
    s.validate(_sample_data())

    assert_true(s.valid)

    s = Schema()

    s.actual_numbers.numeric_string()
    s.validate(_sample_data())

    assert_false(s.valid)


def test_nonblank():
    s = Schema()
    s.random_string.nonblank()
    s.validate(_sample_data())

    assert_true(s.valid)

    s = Schema()
    s.blank_string.nonblank()
    s.validate(_sample_data())

    assert_false(s.valid)


def test_removespaces():
    s = Schema()
    s.awkward_spaces.removespaces()
    s.validate(_sample_data())

    assert_true(s.valid)
    assert_equals(s.awkward_spaces.result, 'openthedoorgetonthefloorrr')


def test_strip():
    s = Schema()
    s.flanking_spaces.strip()
    s.validate(_sample_data())

    assert_true(s.valid)
    assert_equals(s.flanking_spaces.result, 'everybody walk the dinosaur')


def test_lower():
    s = Schema()
    s.upper_case.lower()
    s.validate(_sample_data())

    assert_true(s.valid)
    assert_equals(s.upper_case.result, 'i am the rawrmachine')


def test_upper():
    s = Schema()
    s.lower_case.upper()
    s.validate(_sample_data())

    assert_true(s.valid)
    assert_equals(s.lower_case.result, 'I AM THE QUIET MOUSE')


def test_regex():
    s = Schema()
    s.numeric_string.regex(r'\d')
    s.validate(_sample_data())

    assert_true(s.valid)

    s = Schema()
    s.alpha_string.regex(r'\d')
    s.validate(_sample_data())

    assert_false(s.valid)


def test_canonize():
    s = Schema()
    s.noncanonical_string.canonize()

    s.validate(_sample_data())

    assert_true(s.valid)
    assert_equals(s.noncanonical_string.result,
                  u'there_is_nothing_canonical_aboutmeeeeee333_see?')


def test_slugify():
    s = Schema()
    s.notslugified.slugify()

    s.validate(_sample_data())

    assert_true(s.valid)
    assert_equals(s.notslugified.result,
                  u'i-am-not-slugified--omg-punctu3232ation---------who-submits-data-like-this-anyway')


def test_key_lookup():
    s = Schema()
    s.some_key.key_lookup({'alpha': 1, 'bravo': 2})
    s.some_nonexistent_key.key_lookup({'alpha': 1, 'bravo': 2}, nomatch='fail')
    s.some_nonexistent_key_2.key_lookup({'alpha': 1, 'bravo': 2})

    s.validate(_sample_data())
    assert_false(s.valid)

    assert_true(s.some_key.valid)
    assert_equals(s.some_key.result, 1)

    assert_false(s.some_nonexistent_key.valid)

    assert_true(s.some_nonexistent_key_2.valid)
    assert_equals(s.some_nonexistent_key_2.result, 'delta')



from setuptools import setup, find_packages

setup(
    name="valhalla",
    version="0.0.10",
    description='A slim validation library with a very elegant API designed to afford the least amount of typing.',
    long_description=open('README.md').read(),
    keywords='',
    url='http://github.com/petermelias/valhalla',
    author='Peter M. Elias',
    author_email='petermelias@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=[],
    extras_require={
            'test': ['nose', 'coveralls']
    },
    test_suite='nose.collector',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



# -*- coding: utf-8 -*-

from datetime import datetime, date
from re import compile as re_compile, sub

from valhalla import ValidationError


def _enum(**enums):
    return type('Enum', (), dict(enums.items() + {'items': enums}.items()))


def _explode_int(number):
    return [int(n) for n in str(number)]

_CC_REGEX = _enum(
    VISA=re_compile(r'^4\d{12}(\d{3})?$'),
    MASTERCARD=re_compile(r'^(5[1-5]\d{4}|677189)\d{10}$'),
    AMEX=re_compile(r'^3[47]\d{13}$'),
    DISCOVER=re_compile(r'^(6011|65\d{2})\d{12}$')
)

_CC_TYPE = _enum(
    VISA=u'visa',
    MASTERCARD=u'mastercard',
    AMEX=u'amex',
    DISCOVER=u'discover',
    OTHER=u'other'
)

# accepts a tuple (number, exp_date, csc), returns (long(number),
# date_obj, int(csc), CC_TYPE)


def credit_card(brands=[], _value=(), *args, **kwargs):
    if type(_value) is not tuple:
        raise ValidationError(
            'This validator requires a (number, exp_date, csc) tuple.')

    def _detect_brand(number):
        for t, regex in _CC_REGEX.items.iteritems():
            if regex.match(number):
                return _CC_TYPE.items[t]

        return _CC_TYPE.OTHER

    def _luhn_test(number):
        digits = _explode_int(number)
        # Hehe, yeah.
        return True if not sum(digits[-1::-2] + [sum(_explode_int(n * 2)) for n in digits[-2::-2]]) % 10 else False

    def _mk_date(exp_date):
        exp_date = sub(r'[-/\.\s]', '', exp_date)
        format_dict = {
            4: '%m%y',
            6: '%m%Y'
        }

        if len(exp_date) in (3, 5):
            exp_date = u'0' + exp_date

        try:
            date_format = format_dict[len(exp_date)]
        except KeyError:
            raise ValidationError(
                'Invalid expiration date format. Must be one of : [MMYY, MYY, MYYYY, MMYYYY]')

        return datetime.strptime(exp_date, date_format).date()

    number, exp_date, csc = _value
    if not _luhn_test(number):
        raise ValidationError('The specified credit card number is not valid.')

    cc_type = _detect_brand(number)
    if brands and cc_type not in brands:
        raise ValidationError(
            'The type %s is not an acceptable brand.' % (cc_type))

    exp_date = _mk_date(exp_date)
    if exp_date < date.today():
        raise ValidationError('The credit card is expired.')

    if (cc_type == _CC_TYPE.AMEX and len(csc) != 4) or (cc_type != _CC_TYPE.AMEX and len(csc) != 3):
        raise ValidationError('The CSC code is not valid.')

    return (long(number), exp_date, int(csc), cc_type)



# -*- coding: utf-8 -*-

from importlib import import_module

__all__ = [
    'casting',
    'chrono',
    'collection',
    'logical',
    'money',
    'numerical',
    'strings',
    'web'
]


def lookup(name):
    mod, pre, post = [None for i in xrange(3)]

    for m in __all__:
        mod_name = 'valhalla.filters.%s' % m
        mod = import_module(mod_name)
        if hasattr(mod, name):
            break

    try:
        pre = mod._prehook
        if name in mod._disable_prehook:
            pre = None
    except AttributeError:
        pass

    try:
        post = mod._posthook
        if name in mod._disable_posthook:
            post = None
    except AttributeError:
        pass

    return (getattr(mod, name), pre, post)
